import processing.core.*; 
import processing.xml.*; 

import processing.net.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class DrawTri extends PApplet {



Server s;
Client c;
int pointCnt = 0;
int maxPoint = 50;

byte[] buf = new byte[4];
float[] targetPt = new float[3];
float[] linePts = new float[maxPoint * 3];

float[] newPt = new float[3];
float[] lastPt = new float[3];

PGraphics pg;
boolean isLineEnable = false;
boolean isRotateEnable = false;

float cameraRot = 0.0f;
float triangleRot = 0.0f;

public void setup(){
	size(800, 600, P3D);
	background(0);
	frameRate(24);
	s = new Server(this, 5000);
	resetPt();
	
	background(0,0,0);
	
	pg = createGraphics(width, height, P3D);
	
}

public void draw(){
	loadClientData();
	lights();
	pg.background(0,0,0,30);
	pg.beginDraw();
	pg.colorMode(HSB, 2,2,2);


	if(isRotateEnable){
		pg.camera(cos(cameraRot) * 500, -250, sin(cameraRot) * 500,
		0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);
		cameraRot += 0.003f;
	}else{
		pg.camera(0.0f, 0.0f, 500,
		0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);
	}
	triangleRot += 0.06f;

	pg.pushMatrix();
	pg.scale(250, 250, 250);
	
	pg.stroke(2,0,2, 80);
	pg.line(-1,0,0,1,0,0);
	pg.line(0,-1,0,0,1,0);
	pg.line(0,0,-1,0,0,1);

	lastPt[0] = linePts[0]; lastPt[1] = linePts[1]; lastPt[2] = linePts[2];
	int i;
	int j;
	for(i = 0; i < maxPoint; i++){
		int idx = i * 3;
		newPt[0] = linePts[idx]; 
		newPt[1] = linePts[idx + 1];
		newPt[2] = linePts[idx + 2];

		if(isLineEnable){
			pg.stroke(newPt[0] + 1, newPt[2] + 1, 2.0f, 220);
			pg.line(newPt[0], newPt[1], newPt[2], lastPt[0], lastPt[1], lastPt[2]);
		}

		lastPt[0] = newPt[0];
		lastPt[1] = newPt[1];
		lastPt[2] = newPt[2];

		pg.fill(newPt[0] + 1, newPt[2] + 1, 2.0f, 90);
		pg.noStroke();
		pg.beginShape();
		float triRadius = 0.04f;
		pg.pushMatrix();
		pg.translate(newPt[0], newPt[1], 0);
		pg.rotateZ(triangleRot);
		pg.translate(-newPt[0], -newPt[1], 0);
		pg.vertex(newPt[0] + triRadius * 0.866f, 
				  newPt[1] - triRadius * 0.5f, newPt[2]);
		pg.vertex(newPt[0] - triRadius * 0.866f, 
				  newPt[1] - triRadius * 0.5f, newPt[2]);
		pg.vertex(newPt[0], newPt[1] + triRadius, newPt[2]);
		pg.endShape(CLOSE);
		pg.popMatrix();

		pg.stroke(newPt[0] + 1, newPt[2] + 1, 2.0f);
		float rad;
		for(j = 0; j < 20; j++){
			rad = random(TWO_PI);
			pg.point(newPt[0] + cos(rad) * 0.08f, 
				  newPt[1] + sin(rad) * 0.08f,
				  newPt[2]);
			pg.point(newPt[0] + cos(rad) * 0.1f, 
				  newPt[1] + sin(rad) * 0.1f,
				  newPt[2]);
		}
	}

	pg.popMatrix();

	pg.endDraw();
	image(pg, 0, 0);
}

public void loadClientData(){
	c = s.available();
	if(c != null){
		while(c.available() > 0){
			for(int i= 0; i < 3; i++){
				c.readBytes(buf);
				float f = parseFloatByByte(buf);
				if(checkFloatRange(f)){
					targetPt[i] = f;
					linePts[pointCnt * 3 + i] = f;
				}
			}
			pointCnt++;
			if(pointCnt == maxPoint){
				pointCnt = 0;
			}

			c.readBytes(buf);
			int isLine = parseIntByByte(buf);
			isLineEnable = isLine == 1 ? true : false;
			c.readBytes(buf);
			int isRotate = parseIntByByte(buf);
			isRotateEnable = isRotate == 1 ? true : false;
		}
		c.clear();
	}
}

public float parseFloatByByte(byte[] buffer){
	int parsedInt = parseIntByByte(buffer);
	return Float.intBitsToFloat(parsedInt);
}

public int parseIntByByte(byte[] buffer){
	int ans = 0;
	for(int i = 0; i < 4; i++){
		ans += (buffer[i] & 0xff) << (8 * (3 - i));
	}
	return ans;
}

public void resetPt(){
	float[] vec = {0.0f, 0.0f, 0.0f};
	for(int i = 0; i < 3; i++){
		targetPt[i] = vec[i];
	}
}

public boolean checkFloatRange(float f){
	boolean isOK = true;
	if(Float.isNaN(f) == true || Float.isInfinite(f) == true ||
	f > 1.0f || f < -1.0f){
		isOK = false;
	}
	return isOK;
}

public void serverEvent(Server someServer, Client someClient) {
  println("We have a new client: " + someClient.ip());
}
  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "DrawTri" });
  }
}
